@@ -1,4 +1,4 @@
-# Copyright (c) 2021-2022, InterDigital Communications, Inc
+# Copyright (c) 2021-2024, InterDigital Communications, Inc
 # All rights reserved.
 
 # Redistribution and use in source and binary forms, with or without
@@ -27,7 +27,9 @@
 # OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 # ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-from typing import Any
+import math
+
+from typing import Any, Tuple
 
 import torch
 import torch.nn as nn
@@ -40,15 +42,79 @@ from .gdn import GDN
 __all__ = [
     "AttentionBlock",
     "MaskedConv2d",
+    "CheckerboardMaskedConv2d",
     "ResidualBlock",
     "ResidualBlockUpsample",
     "ResidualBlockWithStride",
+    "conv1x1",
+    "SpectralConv2d",
+    "SpectralConvTranspose2d",
     "conv3x3",
     "subpel_conv3x3",
     "QReLU",
+    "sequential_channel_ramp",
 ]
 
 
+class _SpectralConvNdMixin:
+    def __init__(self, dim: Tuple[int, ...]):
+        self.dim = dim
+        self.weight_transformed = nn.Parameter(self._to_transform_domain(self.weight))
+        del self._parameters["weight"]  # Unregister weight, and fallback to property.
+
+    @property
+    def weight(self) -> Tensor:
+        return self._from_transform_domain(self.weight_transformed)
+
+    def _to_transform_domain(self, x: Tensor) -> Tensor:
+        return torch.fft.rfftn(x, s=self.kernel_size, dim=self.dim, norm="ortho")
+
+    def _from_transform_domain(self, x: Tensor) -> Tensor:
+        return torch.fft.irfftn(x, s=self.kernel_size, dim=self.dim, norm="ortho")
+
+
+class SpectralConv2d(nn.Conv2d, _SpectralConvNdMixin):
+    r"""Spectral 2D convolution.
+
+    Introduced in [Balle2018efficient].
+    Reparameterizes the weights to be derived from weights stored in the
+    frequency domain.
+    In the original paper, this is referred to as "spectral Adam" or
+    "Sadam" due to its effect on the Adam optimizer update rule.
+    The motivation behind representing the weights in the frequency
+    domain is that optimizer updates/steps may now affect all
+    frequencies to an equal amount.
+    This improves the gradient conditioning, thus leading to faster
+    convergence and increased stability at larger learning rates.
+
+    For comparison, see the TensorFlow Compression implementations of
+    `SignalConv2D
+    <https://github.com/tensorflow/compression/blob/v2.14.0/tensorflow_compression/python/layers/signal_conv.py#L61>`_
+    and
+    `RDFTParameter
+    <https://github.com/tensorflow/compression/blob/v2.14.0/tensorflow_compression/python/layers/parameters.py#L71>`_.
+
+    [Balle2018efficient]: `"Efficient Nonlinear Transforms for Lossy
+    Image Compression" <https://arxiv.org/abs/1802.00847>`_,
+    by Johannes Ballé, PCS 2018.
+    """
+
+    def __init__(self, *args: Any, **kwargs: Any):
+        super().__init__(*args, **kwargs)
+        _SpectralConvNdMixin.__init__(self, dim=(-2, -1))
+
+
+class SpectralConvTranspose2d(nn.ConvTranspose2d, _SpectralConvNdMixin):
+    r"""Spectral 2D transposed convolution.
+
+    Transposed version of :class:`SpectralConv2d`.
+    """
+
+    def __init__(self, *args: Any, **kwargs: Any):
+        super().__init__(*args, **kwargs)
+        _SpectralConvNdMixin.__init__(self, dim=(-2, -1))
+
+
 class MaskedConv2d(nn.Conv2d):
     r"""Masked 2D convolution implementation, mask future "unseen" pixels.
     Useful for building auto-regressive network components.
@@ -74,10 +140,36 @@ class MaskedConv2d(nn.Conv2d):
 
     def forward(self, x: Tensor) -> Tensor:
         # TODO(begaintj): weight assigment is not supported by torchscript
-        self.weight.data *= self.mask
+        self.weight.data = self.weight.data * self.mask
         return super().forward(x)
 
 
+class CheckerboardMaskedConv2d(MaskedConv2d):
+    r"""Checkerboard masked 2D convolution; mask future "unseen" pixels.
+
+    Checkerboard mask variant used in
+    `"Checkerboard Context Model for Efficient Learned Image Compression"
+    <https://arxiv.org/abs/2103.15306>`_, by Dailan He, Yaoyan Zheng,
+    Baocheng Sun, Yan Wang, and Hongwei Qin, CVPR 2021.
+
+    Inherits the same arguments as a `nn.Conv2d`. Use `mask_type='A'` for the
+    first layer (which also masks the "current pixel"), `mask_type='B'` for the
+    following layers.
+    """
+
+    def __init__(self, *args: Any, mask_type: str = "A", **kwargs: Any):
+        super().__init__(*args, **kwargs)
+
+        if mask_type not in ("A", "B"):
+            raise ValueError(f'Invalid "mask_type" value "{mask_type}"')
+
+        _, _, h, w = self.mask.size()
+        self.mask[:] = 1
+        self.mask[:, :, 0::2, 0::2] = 0
+        self.mask[:, :, 1::2, 1::2] = 0
+        self.mask[:, :, h // 2, w // 2] = mask_type == "B"
+
+
 def conv3x3(in_ch: int, out_ch: int, stride: int = 1) -> nn.Module:
     """3x3 convolution with padding."""
     return nn.Conv2d(in_ch, out_ch, kernel_size=3, stride=stride, padding=1)
@@ -284,7 +376,7 @@ class QReLU(Function):
         grad_input = grad_output.clone()
         grad_sub = (
             torch.exp(
-                (-ctx.alpha**ctx.beta)
+                (-(ctx.alpha**ctx.beta))
                 * torch.abs(2.0 * input / ctx.max_value - 1) ** ctx.beta
             )
             * grad_output.clone()
@@ -294,3 +386,40 @@ class QReLU(Function):
         grad_input[input > ctx.max_value] = grad_sub[input > ctx.max_value]
 
         return grad_input, None, None
+
+
+def sequential_channel_ramp(
+    in_ch: int,
+    out_ch: int,
+    *,
+    min_ch: int = 0,
+    num_layers: int = 3,
+    interp: str = "linear",
+    make_layer=None,
+    make_act=None,
+    skip_last_act: bool = True,
+    **layer_kwargs,
+) -> nn.Module:
+    """Interleave layers of gradually ramping channels with nonlinearities."""
+    channels = ramp(in_ch, out_ch, num_layers + 1, method=interp).floor().int()
+    channels[1:-1] = channels[1:-1].clip(min=min_ch)
+    channels = channels.tolist()
+    layers = [
+        module
+        for ch_in, ch_out in zip(channels[:-1], channels[1:])
+        for module in [
+            make_layer(ch_in, ch_out, **layer_kwargs),
+            make_act(),
+        ]
+    ]
+    if skip_last_act:
+        layers = layers[:-1]
+    return nn.Sequential(*layers)
+
+
+def ramp(a, b, steps=None, method="linear", **kwargs):
+    if method == "linear":
+        return torch.linspace(a, b, steps, **kwargs)
+    if method == "log":
+        return torch.logspace(math.log10(a), math.log10(b), steps, **kwargs)
+    raise ValueError(f"Unknown ramp method: {method}")
