@@ -1,4 +1,4 @@
-# Copyright (c) 2021-2022, InterDigital Communications, Inc
+# Copyright (c) 2021-2024, InterDigital Communications, Inc
 # All rights reserved.
 
 # Redistribution and use in source and binary forms, with or without
@@ -30,6 +30,7 @@
 Collect performance metrics of published traditional or end-to-end image
 codecs.
 """
+
 import argparse
 import json
 import multiprocessing as mp
@@ -141,7 +142,7 @@ def setup_common_args(parser):
     parser.add_argument(
         "--metrics",
         dest="metrics",
-        default=["psnr", "ms-ssim"],
+        default=["psnr-rgb", "ms-ssim-rgb"],
         nargs="+",
         help="do not return PSNR and MS-SSIM metrics (use for very small images)",
     )
