@@ -1,4 +1,4 @@
-# Copyright (c) 2021-2022, InterDigital Communications, Inc
+# Copyright (c) 2021-2024, InterDigital Communications, Inc
 # All rights reserved.
 
 # Redistribution and use in source and binary forms, with or without
@@ -30,6 +30,7 @@
 Simple plotting utility to display Rate-Distortion curves (RD) comparison
 between codecs.
 """
+
 import argparse
 import json
 import sys
@@ -64,7 +65,7 @@ def parse_json_file(filepath, metric):
         )
 
     try:
-        if metric == "ms-ssim":
+        if "ms-ssim" in metric:
             # Convert to db
             values = np.array(results[metric])
             results[metric] = -10 * np.log10(1 - values)
